#!/usr/bin/perl

use strict;
use DateTime;
use DateTime::Format::Strptime;
use Getopt::Long;
use Net::SNMP;
use Data::Dumper;
use lib "/usr/local/esia/plugins";
use utils qw(%ERRORS);
my $community=undef;
my $critical=60;
my $help=undef;
my $login=undef;
my $password=undef;
my $remote_host=undef;
my $remote_tz=undef;
my $snmp_ver="2c";
my $snmp_port=161;
my $warn=30;
my $debug;
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my ($snmp, $snmperror);
my $local_time="";
my $local_tz="";
my $parameters="";
my $exit_code=0;

sub help {
   print "\nNagios plugin to check the time on a server using SNMP.\n";
   print "GPL licence, (c)2007 Karl Bolingbroke\n";
	print "Version 1.02\n\n";
	print "This SNMP OID is known to work on various versions of Linux and Windoze hosts,\n";
	print "and is known to NOT work on HP-UX 11.11.\n";
   print_usage();
   print <<EOT;
-c, --critical=CRITICAL
   Number of seconds the time can be off before giving critical alert. (Default: 60)
-h, --help
   Print this help message
-H, --hostname=HOST
   Name or IP address of host to check.
-C, --community=COMMUNITY NAME
   Community name for the host's SNMP agent (implies SNMP v1 or v2c with option).
-S, --snmpver=VERSION
   Version of SNMP protocol to use: 1|2c|3. (Default: 2c)
-l, --login=LOGIN
   Login for snmpv3 authentication (implies v3 protocol with MD5)
-p, --port=PORT
-w, --warn=WARN
   Number of seconds the time can be off before giving warning alert. (Default: 30)
-x, --password=PASSWORD
   Password for snmpv3 authentication
-Z, --timezone=TIMEZONE
   Timezone of the remote host.  This is necessary, because Windoze hosts don't report the timezone.

Note:
  The script will return 
    OK if the time on the remote server is < WARNING seconds off from the Nagios server.
    WARNING if the remote server's time is > WARNING and < CRITICAL seconds different.
    CRITICAL if the remote server's time > CRITICAL seconds different.
EOT
}

sub check_options {
   Getopt::Long::Configure ("bundling");
   GetOptions(
        'c:i'   => \$critical,          'critical:i'    => \$critical,
		  'd'     => \$debug,             'debug'         => \$debug,
        'h'     => \$help,              'help'          => \$help,
		  'l:s'   => \$login,             'login:s'       => \$login,
		  'p:i'   => \$snmp_port,         'port:i'        => \$snmp_port,
        'w:i'   => \$warn,              'warn:i'        => \$warn,
		  'x:s'   => \$password,          'password:s'    => \$password,
        'C:s'   => \$community,         'community:s'   => \$community,
        'H:s'   => \$remote_host,       'hostname:s'    => \$remote_host,
        'S:s'   => \$snmp_ver,          'snmpver:s'     => \$snmp_ver,
        'Z:s'   => \$remote_tz,         'timezone:s'    => \$remote_tz
   );
	if (defined ($help)) { help(); exit $ERRORS{"UNKNOWN"}};
	# Check the SNMP parameters
	if ( !defined($community) && (!defined($login) || !defined($password)) ) {
	 	print "Missing SNMP login information!\n";
		print_usage();
		exit $ERRORS{"UNKNOWN"}
	}
	if ( !defined($remote_host) ) {
		print "Missing name of host to check!\n";
		print_usage();
		exit $ERRORS{"UNKNOWN"}
	}
	if ( !defined($remote_tz) ) {
		print "Missing timezone of the remote host!\nThis is required because Windoze hosts don't report their timezone.\n";
		print_usage();
		exit $ERRORS{"UNKNOWN"}
	}
}

sub print_usage {
    print "Usage: check_time_snmp.pl -H <host> -C <SNMP community> -Z <timezone of host> [-l <SNMP login> -x <SNMP password] [-S <SNMP version 1|2c|3>] [-p <port>] [-c <critical>] [-w <warn>]\n";
}

####### MAIN PROGRAM ########
check_options();

if ($snmp_ver == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $remote_host,
			-version    => 'v2c',
			-domain     => $domain,
			-port       => $snmp_port,
			-community  => $community,
            -translate   => [-octetstring => 0],
			-timeout    => 3
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $remote_host,
			-version   => 'v1',
			-domain    => $domain,
			-port      => $snmp_port,
			-community => $community,
            -translate   => [-octetstring => 0],
			-timeout   => 3
		);
}

my $result = $snmp->get_request("1.3.6.1.2.1.25.1.2.0");
if (!defined($result))
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
my $remote_ts = $$result{'1.3.6.1.2.1.25.1.2.0'}; 
my @date = unpack 'n C6 a C2', $remote_ts;
my $formatter = DateTime::Format::Strptime->new(pattern=>"%s");
my $local_ts=DateTime->now(time_zone=>"local",formatter =>$formatter);
$remote_ts=DateTime->new( year=>$date[0], month=>$date[1], day=>$date[2], hour=>$date[3], minute=>$date[4], second=>$date[5], time_zone=>$remote_tz,formatter =>$formatter);

if (defined $debug) { print "$local_ts|$remote_ts"; }
my $diff=int($local_ts)-int($remote_ts);

if (abs($diff) > $critical) {
	print "CRITICAL - $remote_host time is off by $diff seconds!\n";
	exit $ERRORS{"CRITICAL"};
} elsif ($diff > $warn) {
	print "WARNING - $remote_host time is off by $diff seconds!\n";
	exit $ERRORS{"WARNING"};
} else {
	print "OK - $remote_host time differs by $diff seconds.\n";
	exit $ERRORS{"OK"};
}
